import socket
import ssl

import paho.mqtt.client as mqtt

from ..mqtt.Constants import MQTT_USER, MQTT_PASSWORD


class Client:

    __instance = None

    @staticmethod
    def getInstance():
        """ Static access method. """
        if Client.__instance == None:
            Client()
        return Client.__instance

    def __init__(self):
        """ Virtually private constructor. """
        if Client.__instance != None:
            raise Exception("This class is a Singleton!")
        else:
            Client.__instance = self
            self.__connected = False
            self.__callbacks = {}
            self.__paho_client = mqtt.Client(transport="websockets")
            self.__paho_client.ws_set_options(path="/ws")

            context = ssl.create_default_context()
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            context.wrap_socket(s, server_hostname="*.beemo.eu")
            self.__paho_client.tls_set_context(context)

            self.__paho_client.on_disconnect = self.__on_disconnect
            self.__paho_client.on_connect = self.__on_connect
            self.__paho_client.on_message = self.__on_message
       
    def __del__(self):
        self.disconnect()

    def connect(self):
        self.__paho_client.username_pw_set(MQTT_USER, MQTT_PASSWORD)
        self.__paho_client.connect("rabbitmq-data-live.beemo.eu", 443, 60)
        self.__paho_client.loop_start()

    def disconnect(self):
        self.__paho_client.disconnect()
    
    def is_connected(self):
        return self.__connected

    def subscribe(self, topic, callback):
        self.__add_callback(topic, callback)

    def unsubscribe(self, topic, callback):
        self.__remove_callback(topic, callback)

    def publish(self, topic, payload):
        self.__paho_client.publish(topic, payload, qos=2, retain=True)

    def __on_connect(self, client, userdata, flags, rc):
        if rc != 0:
            print("Unexpected connection.")
        else:
            for topic in self.__callbacks:
                self.__paho_client.subscribe(topic)
            self.__connected = True

    def __on_disconnect(self, client, userdata, rc):
        if rc != 0:
            print("Unexpected disconnection.")
        else:
            self.__connected = False

    def __on_message(self, client, userdata, msg):
        topic = str(msg.topic)
        payload = str(msg.payload.decode("utf-8"))
        if topic in self.__callbacks:
            for callback in self.__callbacks[topic]:
                callback(payload)

    def __add_callback(self, topic, callback):
        if self.__has_callback(topic, callback) == False:
            if topic not in self.__callbacks:
                self.__callbacks[topic] = []
            self.__callbacks[topic].append(callback)

    def __remove_callback(self, topic, callback):
        if self.__has_callback(topic, callback) == True:
            self.__callbacks[topic].remove(callback)
            if len(self.__callbacks[topic]) == 0:
                del self.__callbacks[topic]

    def __has_callback(self, topic, callback):
        return topic in self.__callbacks and callback in self.__callbacks[topic]
